@@ -1,34 +1,39 @@
-class ArtifactsMissing(Exception):
+class JenkinsAPIException(Exception):
+    """
+    Base class for all errors
+    """
+
+class ArtifactsMissing(JenkinsAPIException):
     """
     Cannot find a build with all of the required artifacts.
     """
 
-class UnknownJob( KeyError ):
+class UnknownJob( KeyError, JenkinsAPIException):
     """
     Jenkins does not recognize the job requested.
     """
 
-class ArtifactBroken(Exception):
+class ArtifactBroken(JenkinsAPIException):
     """
     An artifact is broken, wrong
     """
 
-class TimeOut( Exception ):
+class TimeOut( JenkinsAPIException ):
     """
     Some jobs have taken too long to complete.
     """
 
-class WillNotBuild(Exception):
+class WillNotBuild(JenkinsAPIException):
     """
     Cannot trigger a new build.
     """
 
-class NoBuildData(Exception):
+class NoBuildData(JenkinsAPIException):
     """
     A job has no build data.
     """
 
-class NoResults(Exception):
+class NoResults(JenkinsAPIException):
     """
     A build did not publish any results.
     """
@@ -38,30 +43,30 @@ class FailedNoResults(NoResults):
     A build did not publish any results because it failed
     """
 
-class BadURL(ValueError):
+class BadURL(ValueError,JenkinsAPIException):
     """
     A URL appears to be broken
     """
 
-class NotFound(Exception):
+class NotFound(JenkinsAPIException):
     """
     Resource cannot be found
     """
 
-class NotAuthorized(Exception):
+class NotAuthorized(JenkinsAPIException):
     """Not Authorized to access resource"""
     # Usually thrown when we get a 403 returned
 
-class NotSupportSCM(Exception):
+class NotSupportSCM(JenkinsAPIException):
     """
     It's a SCM that does not supported by current version of jenkinsapi
     """
 
-class NotConfiguredSCM(Exception):
+class NotConfiguredSCM(JenkinsAPIException):
     """
     It's a job that doesn't have configured SCM
     """
-class NotInQueue(Exception):
+class NotInQueue(JenkinsAPIException):
     """
     It's a job that is not in the queue
     """
