@@ -1,18 +1,23 @@
-from jenkinsapi.exceptions import UnknownJob, NotAuthorized
-from jenkinsapi.fingerprint import Fingerprint
-from jenkinsapi.jenkinsbase import JenkinsBase
-from jenkinsapi.job import Job
-from jenkinsapi.node import Node
-from jenkinsapi.queue import Queue
-from jenkinsapi.view import View
-from jenkinsapi import config
-from utils.urlopener import mkurlopener, mkopener, NoAuto302Handler
-import cookielib
-import logging
 import time
 import urllib
 import urllib2
+import logging
 import urlparse
+import requests
+import StringIO
+import cookielib
+from utils.urlopener import mkurlopener, mkopener, NoAuto302Handler
+
+from jenkinsapi import config
+from jenkinsapi.job import Job
+from jenkinsapi.nodes import Nodes
+from jenkinsapi.node import Node
+from jenkinsapi.queue import Queue
+from jenkinsapi.view import View
+from jenkinsapi.fingerprint import Fingerprint
+from jenkinsapi.jenkinsbase import JenkinsBase
+from jenkinsapi.utils.requester import Requester
+from jenkinsapi.exceptions import UnknownJob, NotAuthorized, JenkinsAPIException
 
 try:
     import json
@@ -32,83 +37,26 @@ class Jenkins(JenkinsBase):
     """
     Represents a jenkins environment.
     """
-    def __init__(self, baseurl, username=None, password=None, proxyhost=None, proxyport=None, proxyuser=None, proxypass=None, formauth=False, krbauth=False):
+    def __init__(self, baseurl, username=None, password=None, requester=None):
         """
-
         :param baseurl: baseurl for jenkins instance including port, str
         :param username: username for jenkins auth, str
         :param password: password for jenkins auth, str
-        :param proxyhost: proxyhostname, str
-        :param proxyport: proxyport, int
-        :param proxyuser: proxyusername for proxy auth, str
-        :param proxypass: proxypassword for proxyauth, str
         :return: a Jenkins obj
         """
         self.username = username
         self.password = password
-        self.proxyhost = proxyhost
-        self.proxyport = proxyport
-        self.proxyuser = proxyuser
-        self.proxypass = proxypass
-        JenkinsBase.__init__(self, baseurl, formauth=formauth, krbauth=krbauth)
+        self.requester = requester or Requester(username, password)
+        JenkinsBase.__init__(self, baseurl)
 
     def _clone(self):
-        return Jenkins(self.baseurl, username=self.username,
-                       password=self.password, proxyhost=self.proxyhost,
-                       proxyport=self.proxyport, proxyuser=self.proxyuser,
-                       proxypass=self.proxypass, formauth=self.formauth, krbauth=self.krbauth)
-
-    def get_proxy_auth(self):
-        return self.proxyhost, self.proxyport, self.proxyuser, self.proxypass
-
-    def get_jenkins_auth(self):
-        return self.username, self.password, self.baseurl
-
-    def get_auth(self):
-        auth_args = []
-        auth_args.extend(self.get_jenkins_auth())
-        auth_args.extend(self.get_proxy_auth())
-        log.debug("auth_args: %s" % auth_args)
-        return auth_args
+        return Jenkins(self.baseurl, username=self.username, password=self.password, requester=self.requester)
 
     def get_base_server_url(self):
-        return self.baseurl[:-(len(config.JENKINS_API))] 
-
-    def get_opener(self):
-        if self.formauth:
-            return self.get_login_opener()
-        if self.krbauth:
-            return self.get_krb_opener()
-        return mkurlopener(*self.get_auth())
-
-    def get_login_opener(self):
-        hdrs = []
-        if getattr(self, '_cookies', False):
-            mcj = cookielib.MozillaCookieJar()
-            for c in self._cookies:
-                mcj.set_cookie(c)
-            hdrs.append(urllib2.HTTPCookieProcessor(mcj))
-        return mkopener(*hdrs)
-
-    def get_krb_opener(self):
-        if not mkkrbopener:
-            raise NotImplementedError('JenkinsAPI was installed without Kerberos support.')
-        return mkkrbopener(self.baseurl)
-
-    def login(self):
-        formdata = dict(j_username=self.username, j_password=self.password,
-                        remember_me=True, form='/')
-        formdata.update(dict(json=json.dumps(formdata), Submit='log in'))
-        formdata = urllib.urlencode(formdata)
-
-        loginurl = urlparse.urljoin(self.baseurl, 'j_acegi_security_check')
-        mcj = cookielib.MozillaCookieJar()
-        cookiehandler = urllib2.HTTPCookieProcessor(mcj)
-
-        urlopen = mkopener(NoAuto302Handler, cookiehandler)
-        res = urlopen(loginurl, data=formdata)
-        self._cookies = [c for c in mcj]
-        return res.getcode() == 302
+        if config.JENKINS_API in self.baseurl:
+            return self.baseurl[:-(len(config.JENKINS_API))]
+        else:
+            return self.baseurl
 
     def validate_fingerprint(self, id):
         obj_fingerprint = Fingerprint(self.baseurl, id, jenkins_obj=self)
@@ -118,7 +66,7 @@ class Jenkins(JenkinsBase):
     def reload(self):
         '''Try and reload the configuration from disk'''
         try:
-            self.hit_url("%(baseurl)s/reload" % self.__dict__)
+            self.requester.get_url("%(baseurl)s/reload" % self.__dict__)
         except urllib2.HTTPError, e:
             if e.code == 403:
                 raise NotAuthorized("You are not authorized to reload this server")
@@ -136,6 +84,14 @@ class Jenkins(JenkinsBase):
     def get_jenkins_obj(self):
         return self
 
+    def get_create_url(self):
+        # This only ever needs to work on the base object
+        return '%s/createItem' % self.baseurl
+
+    def get_nodes_url(self):
+        # This only ever needs to work on the base object
+        return '%s/computer' % self.baseurl
+
     def get_jobs(self):
         """
         Fetch all the build-names on this Jenkins server.
@@ -183,13 +139,13 @@ class Jenkins(JenkinsBase):
         :param config: configuration of new job, xml
         :return: new Job obj
         """
-        headers = {'Content-Type': 'text/xml'}
-        qs = urllib.urlencode({'name': jobname})
-        url = urlparse.urljoin(self.baseurl, "createItem?%s" % qs)
-        request = urllib2.Request(url, config, headers)
-        self.post_data(request, None)
-        newjk = self._clone()
-        return newjk.get_job(jobname)
+        if self.has_job(jobname):
+            raise JenkinsAPIException('Job %s already exists!' % jobname)
+
+        params = {'name': jobname}
+        self.requester.post_xml_and_confirm_status(self.get_create_url(), data=config, params=params)
+        self.poll()
+        return self[jobname]
 
     def copy_job(self, jobname, newjobname):
         """
@@ -198,13 +154,26 @@ class Jenkins(JenkinsBase):
         :param newjobname: name of new job, str
         :return: new Job obj
         """
-        qs = urllib.urlencode({'name': newjobname,
-                               'mode': 'copy',
-                               'from': jobname})
-        copy_job_url = urlparse.urljoin(self.baseurl, "createItem?%s" % qs)
-        self.post_data(copy_job_url, '')
-        newjk = self._clone()
-        return newjk.get_job(newjobname)
+        params = { 'name': newjobname,
+                   'mode': 'copy',
+                   'from': jobname}
+
+        self.requester.post_and_confirm_status(
+            self.get_create_url(),
+            params=params,
+            data='')
+        self.poll()
+        return self[jobname]
+    
+    def build_job(self, jobname, params={}):
+        """
+        Invoke a build by job name
+        :param jobname: name of exist job, str
+        :param params: the job params, dict
+        :return: none
+        """
+        self[jobname].invoke(params=params)
+        return
 
     def delete_job(self, jobname):
         """
@@ -212,10 +181,13 @@ class Jenkins(JenkinsBase):
         :param jobname: name of a exist job, str
         :return: new jenkins_obj
         """
-        delete_job_url = urlparse.urljoin(self._clone().get_job(jobname).baseurl, "doDelete" )
-        self.post_data(delete_job_url, '')
-        newjk = self._clone()
-        return newjk
+        delete_job_url = self[jobname].get_delete_url()
+        response = self.requester.post_and_confirm_status(
+            delete_job_url,
+            data='some random bytes...'
+        )
+        self.poll()
+        return self
 
     def rename_job(self, jobname, newjobname):
         """
@@ -224,19 +196,29 @@ class Jenkins(JenkinsBase):
         :param newjobname: name of new job, str
         :return: new Job obj
         """
-        qs = urllib.urlencode({'newName': newjobname})
-        rename_job_url = urlparse.urljoin(self._clone().get_job(jobname).baseurl, "doRename?%s" % qs)
-        self.post_data(rename_job_url, '')
-        newjk = self._clone()
-        return newjk.get_job(newjobname)
-
-    def iteritems(self):
-        return self.get_jobs()
+        params = {'newName': newjobname}
+        rename_job_url = self[jobname].get_rename_url()
+        response = self.requester.post_and_confirm_status(
+            rename_job_url, params=params, data='')
+        self.poll()
+        return self[newjobname]
 
     def iterkeys(self):
         for info in self._data["jobs"]:
             yield info["name"]
 
+    def iteritems(self):
+        """
+        :param return: An iterator of pairs. Each pair will be (job name, Job object)
+        """
+        return self.get_jobs()
+
+    def items(self):
+        """
+        :param return: A list of pairs. Each pair will be (job name, Job object)
+        """
+        return list(self.get_jobs())
+
     def keys(self):
         return [ a for a in self.iterkeys() ]
 
@@ -244,6 +226,8 @@ class Jenkins(JenkinsBase):
         return "Jenkins server at %s" % self.baseurl
 
     def _get_views(self):
+        log.debug('_get_views: self._data.has_key[views] %s' % 
+                self._data.has_key('views'))
         if not self._data.has_key("views"):
             pass
         else:
@@ -256,6 +240,7 @@ class Jenkins(JenkinsBase):
     def get_view_url(self, str_view_name):
         try:
             view_dict = self.get_view_dict()
+            log.debug('view_dict=%s' % view_dict)
             return view_dict[ str_view_name ]
         except KeyError:
             #noinspection PyUnboundLocalVariable
@@ -273,11 +258,15 @@ class Jenkins(JenkinsBase):
         return View(str_view_url , str_view_name, jenkins_obj=self)
 
     def delete_view_by_url(self, str_url):
-        url = "%s/doDelete" %str_url
-        self.post_data(url, '')
+        url = "%s/doDelete" % str_url
+        response = self.requester.get_url(url, data='')
         self.poll()
         return self
 
+    def get_nodes(self):
+        url = self.get_nodes_url()
+        return Nodes(url, self)
+
     def create_view(self, str_view_name, person=None):
         """
         Create a view
@@ -289,22 +278,22 @@ class Jenkins(JenkinsBase):
         qs = urllib.urlencode({'value': str_view_name})
         viewExistsCheck_url = urlparse.urljoin(url, "viewExistsCheck?%s" % qs)
         log.debug('viewExistsCheck_url=%s' % viewExistsCheck_url)
-        result = self.hit_url(viewExistsCheck_url)
+        result = self.requester.get_url(viewExistsCheck_url)
         log.debug('result=%s' % result)
         # Jenkins returns "<div/>" if view does not exist
-        if len(result) > len('<div/>'): 
+        if len(result) > len('<div/>'):
             log.error('A view "%s" already exists' % (str_view_name))
             return None
         else:
             data = {"mode":"hudson.model.ListView", "Submit": "OK"}
             data['name'] = str_view_name
-            data['json'] = data.copy()
-            log.debug('json data=%s' % data)
-            params = urllib.urlencode(data)
+            # data['json'] = data.copy()
+            # log.debug('json data=%s' % data)
+            # params = urllib.urlencode(data)
             try:
                 createView_url = urlparse.urljoin(url, "createView")
                 log.debug('createView_url=%s' % createView_url)
-                result = self.post_data(createView_url, params)
+                result = self.requester.post_url(createView_url, data)
             except urllib2.HTTPError, e:
                 log.debug("Error post_data %s" % createView_url)
                 log.exception(e)
@@ -326,16 +315,17 @@ class Jenkins(JenkinsBase):
         qs = urllib.urlencode({'value': str_view_name})
         viewExistsCheck_url = urlparse.urljoin(url, "viewExistsCheck?%s" % qs)
         log.debug('viewExistsCheck_url=%s' % viewExistsCheck_url)
-        result = self.hit_url(viewExistsCheck_url)
+        result = self.requester.get_url(viewExistsCheck_url)
         log.debug('result=%s' % result)
         # Jenkins returns "<div/>" if view does not exist
-        if len(result) == len('<div/>'): 
+        if len(result) == len('<div/>'):
             log.error('A view the name "%s" does not exist' % (str_view_name))
-            return False 
+            return False
         else:
             self.delete_view_by_url(urlparse.urljoin(url, 'view/%s' % str_view_name))
             # We changed Jenkins config - need to update ourself
             self.poll()
+            # TODO: add check here that the view actually been deleted
             return True
 
     def __getitem__(self, jobname):
