@@ -77,7 +77,7 @@ class View(JenkinsBase):
                 log.error('Job "%s" is not known to Jenkins' % str_job_name)
                 return False
 
-        job = self.jenkins_obj.get_job(str_job_name)
+            job = self.jenkins_obj.get_job(str_job_name)
         
         jobs = self._data.setdefault('jobs', [])
         jobs.append({'name': job.name, 'url': job.baseurl})
@@ -120,9 +120,3 @@ class View(JenkinsBase):
                 
     def get_nested_view_dict(self):
         return dict( self._get_nested_views() )
-
-    def id(self):
-        """
-        Calculate an ID for this object.
-        """
-        return "%s.%s" % ( self.className, self.name )
